import React from 'react';
import ReactDOM from 'react-dom';

var dateFormat = require('dateformat');

import BookingBigCalendar from './BookingBigCalendar';
import BookingModal from './BookingModal';

class AdvancedBookingWindow extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      bookings: [],
      startTime: this.props.dataStartTime,
      endTime: this.props.dataEndTime,
      oldestBooking: null,
      modalIsOpen: false,
      modalBookingId: null,
      modalTitle: '',
      modalStateSelect: '',
      modalStartDate: '',
      modalStartTime: '',
      modalEndDate: '',
      modalEndTime: '',
      statusMessage: '',
      statusClass: 'alert',
    };
  }

  componentDidMount() {
    // Get the list of bookings from server.
    this.getBookings();
    if (this.props.dataRefreshInterval > 0) {
      setInterval(this.getBookings, this.props.dataRefreshInterval);
    }
  }

  getBookings = () => {
    // Fetch the bookings from server and parse them.
    nextpath_client.getBookings(this.props.dataUnitId, this.props.dataEventType, this.state.startTime, this.state.endTime, this.state.oldestBooking, (bookings) => (
      this.parseBookings(bookings)
    ));
  }

  parseBookings(bookings) {
    if (bookings.hasOwnProperty('data')) {
      var newBookings = nextpath_helpers.parseBookings(bookings);
      // Update the bookings list if we received new booking from the server.
      if (newBookings.length > 0) {
        this.updateBookings(newBookings);
      }
    }
  }

  updateBookings = (bookings) => {
    this.setState({
      // Merge the bookings, but eliminate duplicate entries.
      bookings: nextpath_helpers.removeDuplicateBookings(bookings.concat(this.state.bookings)),
      // Save the created property of last booking, we will need it to refresh
      // the booking list.
      oldestBooking: Math.round(new Date(bookings[bookings.length - 1].changed).getTime() / 1000),
    })
  }

  loadBookingIntoModal = (booking, slot) => {
    if (booking == null) {
      // We create a new booking.
      this.setState({
        modalBookingId: null,
        modalTitle: Drupal.t('Create booking'),
        modalStateSelect: '',
        modalStartDate: dateFormat(slot.start, 'yyyy-mm-dd'),
        modalStartTime: dateFormat(slot.start, 'HH:MM'),
        modalEndDate: dateFormat(slot.end, 'yyyy-mm-dd'),
        modalEndTime: dateFormat(slot.end, 'HH:MM'),
      });
    }
    else if (slot == null) {
      // We edit an existing booking.
      this.setState({
        modalBookingId: booking.id,
        modalTitle: Drupal.t('Edit booking'),
        modalStateSelect: booking.stateId,
        modalStartDate: dateFormat(booking.start, 'yyyy-mm-dd'),
        modalStartTime: dateFormat(booking.start, 'HH:MM'),
        modalEndDate: dateFormat(booking.end, 'yyyy-mm-dd'),
        modalEndTime: dateFormat(booking.end, 'HH:MM'),
      });
    }
  }

  // Handle the DOM update of form elements.
  handleModalSelectChange = (e) => {
    this.setState({modalStateSelect: e.target.value});
  }
  handleStartDateChange = (e) => {
    this.setState({ modalStartDate: e.target.value });
  };
  handleStartTimeChange = (e) => {
    this.setState({ modalStartTime: e.target.value });
  };
  handleEndDateChange = (e) => {
    this.setState({ modalEndDate: e.target.value });
  };
  handleEndTimeChange = (e) => {
    this.setState({ modalEndTime: e.target.value });
  };

  handleBookingSave = () => {
    var error = nextpath_helpers.validateBooking(this.state.modalStartDate, this.state.modalStartTime, this.state.modalEndDate, this.state.modalEndTime, this.state.modalStateSelect);
    if (error.length == 0) {
      // If no error, go on with saving.
      var startDate = new Date(this.state.modalStartDate + ' ' + this.state.modalStartTime);
      var endDate = new Date(this.state.modalEndDate + ' ' + this.state.modalEndTime);
      // Pass the form values to the helper function, which will prepare the
      // JSON.
      const b = nextpath_helpers.createBooking(this.props.dataUnitId, this.props.dataEventType, this.state.modalStateSelect, nextpath_helpers.formatDate(startDate), nextpath_helpers.formatDate(endDate), this.state.modalBookingId);
      // After passing the booking to the server, we need to update the state of
      // our component.
      nextpath_client.createBooking(b, this.props.dataEventType, (reply) => (
        this.handleBookingAfterSave(b, reply)
      ));
      this.setState({
        statusClass: 'alert',
        statusMessage: '',
      });
      this.closeModal();
    }
    else {
      // Display the error messages.
      this.setState({
        statusClass: 'alert alert-danger',
        statusMessage: error.join(', '),
      });
    }
  };

  handleBookingDelete = () => {
    // Display a confirmation window for the delete.
    var confirmResult = confirm(Drupal.t('Do you really want to delete the booking?'));
    if (confirmResult == true) {
      nextpath_client.deleteBooking(this.state.modalBookingId, this.props.dataEventType, (reply) => (
        this.handleBookingAfterDelete(this.state.modalBookingId)
      ));
    }
    this.closeModal();
  }

  handleBookingAfterSave = (booking, reply) => {
    // We made an update of a booking, update the state as well.
    if (booking.data.hasOwnProperty('id')) {
      // Find the index of the booking we need to update in the state.
      var index = this.state.bookings.findIndex(element => element.id === booking.data.id)
      // Get the states related to the booking updated.
      var states = nextpath_helpers.getStatesFromRelation(reply);
      // Update the state.
      this.setState(({bookings: bookings}) => ({
        bookings: [
          // Keep everthing before our element, untouched.
          ...bookings.slice(0,index),
          // Update the lement.
          {
            ...bookings[index],
            start: new Date(nextpath_helpers.removeTimezone(reply.data.attributes.event_dates.value)),
            end: new Date(nextpath_helpers.removeTimezone(reply.data.attributes.event_dates.end_value)),
            stateId: reply.data.relationships.event_state_reference.data.id,
            title: states[reply.data.relationships.event_state_reference.data.id].name,
            color: states[reply.data.relationships.event_state_reference.data.id].color,
          },
          // Keep everthing after our element, untouched.
          ...bookings.slice(index+1),
        ]
      }));
    }
    // We created a new booking, load the new bookings from server.
    else {
      this.getBookings();
    }
  }

  handleBookingAfterDelete = (id) => {
    // Find the index of the booking we need to delete from the state.
    var index = this.state.bookings.findIndex(element => element.id === id)
    // Update the state.
    this.setState(({bookings: bookings}) => ({
      bookings: [
        // Keep everthing before our element, untouched.
        ...bookings.slice(0,index),
        // Keep everthing after our element, untouched.
        ...bookings.slice(index+1),
      ]
    }));
  }

  // We edit a booking, load the content into modal and show the modal.
  handleSelectBooking = (booking) => {
    this.loadBookingIntoModal(booking, null);
    this.openModal();
  }

  // We create a booking, load the default content into modal and show the
  // modal.
  handleSelectSlot = ({ start, end }) => {
    this.loadBookingIntoModal(null, { start, end });
    this.openModal();
  }

  handleRangeChange = (range) => {
    // Get the start and end time from the range selected.
    var newStart = null;
    var newEnd = null;
    if (range.hasOwnProperty('start') && range.hasOwnProperty('end')) {
      newStart = nextpath_helpers.formatDate(range.start);
      // Calendar is returning time 00:00 for the ned date, let's increase the
      // end date by 1 to cover the whole day.
      newEnd = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range.end, 1));
    }
    else if (Array.isArray(range)) {
      newStart = nextpath_helpers.formatDate(range[0]);
      // Calendar is returning time 00:00 for the ned date, let's increase the
      // end date by 1 to cover the whole day.      
      newEnd = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range[range.length - 1], 1));
    }

    // If start time of the range is smaller then the start time from the state,
    // update the state to the new start time.
    // If end time of the range is bigger then the end time from the state,
    // update the state to the new end time.
    // After state is uodated, check for new bookings.
    if (newStart != null && newEnd != null) {
      this.setState(
        {
          startTime: newStart < this.state.startTime ? newStart : this.state.startTime,
          endTime: newEnd > this.state.endTime ? newEnd : this.state.endTime,
        },
        () => this.getBookings()
      );
    }
  }

  openModal = () => {
    this.setState({modalIsOpen: true});
  }

  closeModal = () => {
    // On modal close, cleanup the status messages.
    this.setState({
      modalIsOpen: false,
      statusClass: 'alert',
      statusMessage: '',
    });
  }

  render() {
    return (
      <div>
        <div>
          <BookingBigCalendar
            bookings={this.state.bookings}
            culture={this.state.dataLanguage}
            calendarDefaultView={this.props.calendarDefaultView}
            handleSelectBooking={this.handleSelectBooking}
            handleSelectSlot={this.handleSelectSlot}
            handleRangeChange={this.handleRangeChange}
          />
        </div>
        <div>
          <BookingModal
            modalIsOpen={this.state.modalIsOpen}
            modalTitle={this.state.modalTitle}
            modalStateSelect={this.state.modalStateSelect}
            dataStates={this.props.dataStates}
            modalStartDate={this.state.modalStartDate}
            modalStartTime={this.state.modalStartTime}
            modalEndDate={this.state.modalEndDate}
            modalEndTime={this.state.modalEndTime}
            statusClass={this.state.statusClass}
            statusMessage={this.state.statusMessage}
            handleModalSelectChange={this.handleModalSelectChange}
            handleStartDateChange={this.handleStartDateChange}
            handleStartTimeChange={this.handleStartTimeChange}
            handleEndDateChange={this.handleEndDateChange}
            handleEndTimeChange={this.handleEndTimeChange}
            handleBookingSave={this.handleBookingSave}
            handleBookingDelete={this.handleBookingDelete}
            closeModal={this.closeModal}
          />
        </div>
      </div>
    );
  }
}

export default AdvancedBookingWindow;
